#include "SsPreCompile.h" //QObjectHelper

#include "Sstm.h" //Sstm
#include "ui_Sstm.h"
#include "TunnelStatusChecker.h" //TunnelStatusChecker

Sstm::Sstm(QWidget *parent) :QMainWindow(parent),ui(new Ui::Sstm)
{
    ui->setupUi(this);

    connect(&newTunnelInfoDialog,SIGNAL(accepted()),this,SLOT(AddTunnel())); //输入完了隧道信息之后，添加隧道。
    connect(&editTunnelDialog,SIGNAL(accepted()),this,SLOT(EditTunnel())); //输入完了隧道信息之后，修改隧道。
    connect (&stngDlg,SIGNAL(accepted()),this,SLOT(SaveConfig())); //设置对话框确认关闭，则保存设置信息。

    showTray(); //显示托盘图标。

    loadConfig(); //载入配置信息。

    startAutoStartTunnels(); //启动所有应当自动启动的隧道。
} //Sstm::Sstm(QWidget *parent) :QMainWindow(parent),ui(new Ui::Sstm)

Sstm::~Sstm()
{
    delete ui;
}

/*!
 * \brief Sstm::PopNewTunnelDialog 弹出“新隧道”对话框。
 */
void Sstm::PopNewTunnelDialog()
{
    newTunnelInfoDialog.show(); //显示对话框。

    return;
} //void Sstm::PopNewTunnelDialog() //!<弹出“新隧道”对话框。

/*!
 * \brief Sstm::AddTunnel 输入完了隧道信息之后，添加隧道。
 */
void Sstm::AddTunnel()
{
    QString CrtSrvrIp=newTunnelInfoDialog.GetServerIp(); //获取服务器IP。
    quint16 CrtSrvrPrt=newTunnelInfoDialog.GetServerPort(); //获取服务器端口号。
    quint16 CrtLclPrt=newTunnelInfoDialog.GetLocalPort(); //获取本地监听的端口号。
    QString currentUserName=newTunnelInfoDialog.getUserName(); //获取用户名。
    bool autoRestart=newTunnelInfoDialog.isAutoRestart(); //是否自动重启。
    bool autoCheckTunnelStatus=newTunnelInfoDialog.isAutoCheckTunnelStatus(); //是否自动检查隧道状态。
    bool autoStart=newTunnelInfoDialog.isAutoStart(); //是否自动启动。

    QVariantMap CrtVrt; //将此文件转换成QVariant。

    CrtVrt.insert("ServerPort",CrtSrvrPrt); //设置服务器端口号。
    CrtVrt.insert("ServerIP",CrtSrvrIp); //设置服务器IP。
    CrtVrt.insert("LocalPort",CrtLclPrt); //设置本地监听端口号。
    CrtVrt.insert ("UserName",currentUserName); //设置用户名。
    CrtVrt.insert ("AutoRestart",autoRestart); //是否自动重启。
    CrtVrt.insert ("AutoCheckTunnelStatus",autoCheckTunnelStatus); //是否自动检查隧道状态。
    CrtVrt.insert ("AutoStart",autoStart); //是否自动启动。


    TnlLstVrnt << CrtVrt; //添加到隧道信息列表中。

    ui->TnlLstlistWidget->addItem(CrtSrvrIp); //显示一个条目到界面上。


    //保存到配置文件中：
    SaveConfig(); //保存配置信息。
} //void Sstm::AddTunnel() //!<输入完了隧道信息之后，添加隧道。

/*!
 * \brief Sstm::SaveConfig 保存配置信息。
 */
void Sstm::SaveConfig()
{
    QSettings settings("com.stupidbeauty.sstm","Sstm"); //用来保存信息的工具。

    settings.setValue("TunnelList",TnlLstVrnt); //保存。

    qDebug() << __FILE__ << __LINE__ << __func__ << tr("Tunnel list:") << TnlLstVrnt; //Debug.

    settings.beginGroup("sstm"); //开始分组。

    QVariant ttsServerAddress=QVariant(stngDlg.getTtsServerAddress()); //获取TTS服务器的地址。

    settings.setValue("ttsServerAddress",ttsServerAddress); //tts服务器的地址.

    QVariant ttsServerPort=QVariant(stngDlg.getTtsServerPort()); //获取TTS服务器的端口号。

    settings.setValue("ttsServerPort",ttsServerPort); //tts server's port.

    ttsClient.setServerAddress (stngDlg.getTtsServerAddress ()); //设置TTS服务器的地址。
    ttsClient.setServerPort (stngDlg.getTtsServerPort ()); //设置TTS服务器的端口号。


    settings.endGroup(); //关闭分组。

    settings.sync(); //同步。


    return;
} //void Sstm::SaveConfig(QString JsnStr2Sv) //!<保存配置信息。

/*!
 * \brief Sstm::LoadConfig 载入配置信息。
 */
void Sstm::loadConfig()
{
    QSettings settings("com.stupidbeauty.sstm","Sstm"); //用来保存信息的工具。

    TnlLstVrnt=settings.value("TunnelList").toList(); //读取。

    QVariant CrtTnlIfo; //当前隧道信息对象。
    foreach(CrtTnlIfo,TnlLstVrnt) //一个个地显示。
    {
        QString CrtSrvrIp=CrtTnlIfo.toMap().value("ServerIP").toString(); //获取服务器IP。
        quint16 CrtSrvrPrt=CrtTnlIfo.toMap().value("ServerPort").toUInt(); //获取服务器端口号。
        quint16 CrtLclPrt=CrtTnlIfo.toMap().value("LocalPort").toUInt(); //获取本地监听的端口号。
        QString crtUserName=CrtTnlIfo.toMap ().value ("UserName").toString (); //获取用户名。
        bool autoRestart=CrtTnlIfo.toMap ().value ("AutoRestart").toBool (); //获取自动重启选项。
        bool autoCheckTunnelStatus=CrtTnlIfo.toMap ().value ("AutoCheckTunnelStatus").toBool (); //获取自动检查隧道状态选项。
        bool autoStart=CrtTnlIfo.toMap ().value ("AutoStart").toBool (); //获取自动启动选项。

        Tunnel * currentTunnel=new Tunnel(CrtSrvrIp,CrtSrvrPrt,CrtLclPrt,crtUserName); //构造隧道对象。
        currentTunnel->setAutoRestart(autoRestart); //设置是否要自动重启。
        currentTunnel->setAutoCheckTunnelStatus(autoCheckTunnelStatus); //设置是否要自动检查隧道状态。
        currentTunnel->setAutoStart(autoStart); //设置是否要自动启动。

        tunnelList << currentTunnel; //加入到列表中。

        qDebug() << __FILE__ << __LINE__ << __func__ << tr("Tunnel list:") << TnlLstVrnt << tr("List length:") << TnlLstVrnt.length() << tr("Server port:") << CrtSrvrPrt << tr("Local port:") << CrtLclPrt; //Debug.

        ui->TnlLstlistWidget->addItem(QString("%1@%2:%3/%4").arg(crtUserName).arg (CrtSrvrIp).arg (CrtSrvrPrt).arg (CrtLclPrt)); //添加一个条目。
    } //foreach(CrtTnlIfo,TnlLstVrnt) //一个个地显示。

    settings.beginGroup("sstm"); //主程序。

    QString ttsServerAddress=settings.value("ttsServerAddress",QVariant("192.168.2.113")).toString (); //获取tts服务器的地址.

    ttsClient.setServerAddress(ttsServerAddress); //设置服务器地址。
    stngDlg.setTtsServerAddress(ttsServerAddress); //设置TTS服务器地址。

    quint16 ttsServerPort=settings.value("ttsServerPort",QVariant(11245)).toUInt (); //获取tts服务器的端口号。

    ttsClient.setServerPort(ttsServerPort); //设置服务器端口号。
    stngDlg.setTtsServerPort(ttsServerPort); //设置TTS服务器的端口号。

    settings.endGroup(); //关闭分组。
} //void Sstm::LoadConfig() //!<载入配置信息。

/*!
 * \brief Sstm::restartTunnel Restart tunnel.
 */
void Sstm::restartTunnel()
{
    int CrtIdx=ui->TnlLstlistWidget->currentRow(); //获取当前条目的下标。

    restartTunnelOjbect(CrtIdx); //restart 隧道对象。
} //void Sstm::restartTunnel()

/*!
 * \brief Sstm::StartTunnel 启动隧道。
 */
void Sstm::StartTunnel()
{
    int CrtIdx=ui->TnlLstlistWidget->currentRow(); //获取当前条目的下标。

    startTunnelOjbect(CrtIdx); //启动隧道对象。
} //void Sstm::StartTunnel() //!<启动隧道。

/*!
 * \brief Sstm::PopEditTunnelDialog 弹出编辑隧道信息对话框。
 */
void Sstm::PopEditTunnelDialog()
{
    int CrtIdx=ui->TnlLstlistWidget->currentRow(); //获取当前条目的下标。

    if (CrtIdx>=0) //The index is valid.
    {
        QVariant CrtSrvIfo=TnlLstVrnt[CrtIdx]; //获取当前服务器的信息。

        QString CrtSrvrIp=CrtSrvIfo.toMap().value("ServerIP").toString(); //获取服务器IP。
        quint16 CrtSrvrPrt=CrtSrvIfo.toMap().value("ServerPort").toUInt(); //获取服务器端口号。
        quint16 CrtLclPrt=CrtSrvIfo.toMap().value("LocalPort").toUInt(); //获取本地监听的端口号。
        QString userName=CrtSrvIfo.toMap ().value("UserName").toString (); //获取用户名。
        bool autoRestart=CrtSrvIfo.toMap ().value ("AutoRestart").toBool (); //是否自动重启。
        bool autoStart=CrtSrvIfo.toMap ().value ("AutoStart").toBool (); //是否自动启动。
        bool autoCheckTunnelStatus=CrtSrvIfo.toMap().value("AutoCheckTunnelStatus").toBool(); //是否自动检查隧道状态。

        qDebug() << __FILE__ << __LINE__ << __func__  << tr("user name:") << userName << tr("Auto restart:") << autoRestart; //Debug.



        editTunnelDialog.SetServerIP(CrtSrvrIp); //设置服务器IP。
        editTunnelDialog.SetServerPort(CrtSrvrPrt); //设置服务器端口号。
        editTunnelDialog.SetLocalPort(CrtLclPrt); //设置本地端口号。
        editTunnelDialog.setUserName(userName); //设置用户名。
        editTunnelDialog.setAutoRestart(autoRestart); //设置是否自动重启。
        editTunnelDialog.setAutoStart(autoStart); //设置是否自动启动。
        editTunnelDialog.setAutoCheckTunnelStatus(autoCheckTunnelStatus); //设置是否自动检查隧道状态。

        editTunnelDialog.show(); //显示。

    } //if (CrtIdx>=0) //The index is valid.
    else //The index is not valid.
    {

        QMessageBox::information(this,tr("No tunnel chosen"),tr("Select a tunnel and then press Edit."),QMessageBox::Ok); //Show the error message.
    } //else //The index is not valid.



} //void Sstm::PopEditTunnelDialog() //!<弹出编辑隧道信息对话框。


/*!
 * \brief Sstm::EditTunnel 输入完了隧道信息之后，修改隧道。
 */
void Sstm::EditTunnel()
{
    QString CrtSrvrIp=editTunnelDialog.GetServerIp(); //获取服务器IP。
    quint16 CrtSrvrPrt=editTunnelDialog.GetServerPort(); //获取服务器端口号。
    quint16 CrtLclPrt=editTunnelDialog.GetLocalPort(); //获取本地监听的端口号。
    QString currentUserName=editTunnelDialog.getUserName(); //获取用户名。
    bool autoRestart=editTunnelDialog.isAutoRestart(); //是否自动重启。
    bool autoCheckTunnelStatus=editTunnelDialog.isAutoCheckTunnelStatus(); //是否自动检查隧道状态。
    bool autoStart=editTunnelDialog.isAutoStart(); //是否自动启动。

    int CrtIdx=ui->TnlLstlistWidget->currentRow(); //获取当前条目的下标。


    QVariantMap CrtVrt; //将此文件转换成QVariant。

    CrtVrt.insert("ServerPort",CrtSrvrPrt); //设置服务器端口号。
    CrtVrt.insert("ServerIP",CrtSrvrIp); //设置服务器IP。
    CrtVrt.insert("LocalPort",CrtLclPrt); //设置本地监听端口号。
    CrtVrt.insert ("UserName",currentUserName); //设置用户名。
    CrtVrt.insert ("AutoRestart",autoRestart); //是否自动重启。
    CrtVrt.insert ("AutoCheckTunnelStatus",autoCheckTunnelStatus); //是否自动检查隧道状态。
    CrtVrt.insert ("AutoStart",autoStart); //是否自动启动。


    qDebug() << __FILE__ << __LINE__ << __func__ << tr("Current variant:") << CrtVrt; //Debug.


    TnlLstVrnt[CrtIdx]=CrtVrt; //替换隧道信息列表中的对象。


    QString currentUserNameAtCrtSrvrIp=currentUserName+"@"+CrtSrvrIp; //构造完整的隧道标识 。

    ui->TnlLstlistWidget->takeItem(CrtIdx); //删除对应的显示条目。
    ui->TnlLstlistWidget->insertItem(CrtIdx,currentUserNameAtCrtSrvrIp); //重新将条目插入。


    //保存到配置文件中：
    SaveConfig(); //保存配置信息。
} //void Sstm::EditTunnel() //!<输入完了隧道信息之后，修改隧道。

/*!
 * \brief Sstm::reportTunnelFinish 报告隧道进程的终止事件。隧道进程结束，则报告状态。
 * \param extCd 退出代码。
 */
void Sstm::reportTunnelFinish(int extCd)
{
    Q_UNUSED(extCd); //不使用这个参数。

    QProcess * crtPrc=(QProcess *)sender(); //获取信号的发送者。

    QByteArray prcOtptBtAr=crtPrc->readAll (); //全部内容读取。

    QString prcOtpt=QString::fromUtf8 (prcOtptBtAr); //转换成字符串。


    QByteArray prcOtptBtArErr=crtPrc->readAllStandardError (); //全部内容读取。

    QString prcOtptErr=QString::fromUtf8 (prcOtptBtArErr); //转换成字符串。

    prcOtpt+=prcOtptErr; //接到后面。


    QByteArray procStdErrBtAr=crtPrc->readAllStandardError (); //读取全部错误流数据。

    QString prcStdErr=QString::fromUtf8 (procStdErrBtAr); //转换成字符串。

    prcOtpt+=prcStdErr; //追加。

    qDebug() << __FILE__ << __LINE__ << __func__ ; //显示。
    ttsClient.saySentence(tr("Tunnel is broken.")); //说话，隧道已断开。

    errIfoDlg.show(); //显示错误信息对话框。
    errIfoDlg.setErrorInfo(prcOtpt); //设置错误信息。

    QProcess * quitingProcess=(QProcess *)sender(); //获取发送该信号的进程。

    int tunnelInfoIndex=processTunnelInfoIndexMap[quitingProcess]; //获取到该进程对应的隧道信息的下标。

    tunnelStatusList[tunnelInfoIndex]=Stopped; //记录状态，处于停止状态。
} //void Sstm::reportTunnelFinish(int extCd) //!<隧道进程结束，则报告状态。

/*!
 * \brief Sstm::reportTunnelError 进程出错，则报告错误。
 * \param err 错误码。
 */
void Sstm::reportTunnelError(QProcess::ProcessError err)
{

    errIfoDlg.show(); //显示错误信息对话框。
    errIfoDlg.setErrorInfo(tr("Tunnel error:%1").arg (err)); //设置错误信息。

    qDebug() << __FILE__ << __LINE__ << __func__ ; //Debug.

    ttsClient.saySentence(tr("Tunnel is broken.")); //说话，隧道已断开。

    QProcess * quitingProcess=(QProcess *)sender(); //获取发送该信号的进程。

    int tunnelInfoIndex=processTunnelInfoIndexMap[quitingProcess]; //获取到该进程对应的隧道信息的下标。

    tunnelStatusList[tunnelInfoIndex]=Stopped; //记录状态，处于停止状态。
} //void Sstm::reportTunnelError(QProcess::ProcessError err) //!<进程出错，则报告错误。

/*!
   * \brief SBrowserWindow::showSettingDialog 显示设置对话框。
   */
void Sstm::showSettingDialog()
{
    stngDlg.show(); //显示对话框。
} //void SBrowserWindow::showSettingDialog() //!<显示设置对话框。

/*!
 * \brief Sstm::quit 退出。
 */
void Sstm::quit()
{
    QApplication::quit (); //退出程序。
} //void Sstm::quit() //!<退出。

/*!
 * \brief AptF::showTray 显示托盘图标。
 */
void Sstm::showTray()
{
    QAction * QuitAct=new QAction(this); //退出按钮。
    QuitAct->setText (tr("Quit")); //设置文字。

    connect (QuitAct,&QAction::triggered,this,&Sstm::quit); //按退出按钮，则退出。
    connect (&TrayIcn,&QSystemTrayIcon::activated,this,&Sstm::AsesShow); //点击左键时，显示主窗口。

    TrayMn.addAction (QuitAct); //退出按钮。

    TrayIcn.setIcon (QIcon(":/icon.png")); //设置托盘图标的图片。
    TrayIcn.show(); //显示托盘图标。
} //void AptF::showTray()

/*!
 * \brief XyRunner::AsesShow 点击左键时，显示主窗口。
 * \param ActRsn 激活原因。
 */
void Sstm::AsesShow(QSystemTrayIcon::ActivationReason ActRsn)
{
    if (ActRsn==QSystemTrayIcon::Trigger) //左键。
    {
        show(); //显示。
    } //if (ActRsn==QSystemTrayIcon::Trigger) //左键。
} //void XyRunner::AsesShow(QSystemTrayIcon::ActivationReason ActRsn) //!<点击左键时，显示主窗口。

/*!
 * \brief Sstm::asesRestart 进程出错，则评估一下是否要重启。
 * \param errorCode 错误码。
 */
void Sstm::asesRestart()
{
    QProcess * quitingProcess=(QProcess *)sender(); //获取发送该信号的进程。

    qDebug() << __FILE__ << __LINE__ << __func__ << tr("Quiting process:") << quitingProcess << tr("Time:") << QTime::currentTime ().toString (); //Debug.

    int tunnelInfoIndex=processTunnelInfoIndexMap[quitingProcess]; //获取到该进程对应的隧道信息的下标。

    QVariant CrtSrvIfo=TnlLstVrnt[tunnelInfoIndex]; //获取当前服务器的信息。

    bool autoRestart=CrtSrvIfo.toMap ().value ("AutoRestart").toBool (); //获取自动重启选项。

    if (autoRestart) //应当自动重启。
    {
        if (tunnelStatusList[tunnelInfoIndex]==Stopped) //是处于停止状态。
        {
            startTunnelOjbect(tunnelInfoIndex); //启动隧道对象。
        } //if (tunnelStatusList[tunnelInfoIndex]==Stopped) //是处于停止状态。
    } //if (autoRestart) //应当自动重启。
} //void Sstm::asesRestart(QProcess::ProcessError errorCode)

/*!
 * \brief Sstm::updateButtonStatus 更新按钮的状态。
 * \param currentRow 当前行。
 */
void Sstm::updateButtonStatus(int currentRow)
{
    if (tunnelStatusList.contains (currentRow)) //记录了此隧道 的状态。
    {
        if (tunnelStatusList[currentRow]==Running) //处于运行状态。
        {
            ui->startTunnelpushButton->setEnabled (false); //不可按。
        } //if (tunnelStatusList[currentRow]==Running) //处于运行状态。
        else //未处于运行状态。
        {
            ui->startTunnelpushButton->setEnabled (true); //可按。
        } //else //未处理运行状态。
    } //if (tunnelStatusList.contains (currentRow)) //记录了此隧道 的状态。
    else //未记录此隧道的状态。
    {
        ui->startTunnelpushButton->setEnabled (true); //可按。
    } //else //未记录此隧道的状态。
} //void Sstm::updateButtonStatus(int currentRow)

/*!
 * \brief Sstm::startAutoStartTunnels 启动所有应当自动启动的隧道。
 */
void Sstm::startAutoStartTunnels()
{
    int currentTunnelIndex=0; //当前隧道的下标。

    for(Tunnel * currentTunnel:tunnelList) //一个个地检查是否要自动启动。
    {
        if (currentTunnel->getAutoStart()) //应当自动启动。
        {
            startTunnelOjbect(currentTunnelIndex); //启动这个隧道对象。
        } //if (currentTunnel->isAutoStart()) //应当自动启动。

        currentTunnelIndex++; //下标增加。
    } //for(Tunnel * currentTunnel:tunnelList) //一个个地检查是否要自动启动。
} //void Sstm::startAutoStartTunnels()

/*!
 * \brief Sstm::restartTunnelOjbect restart 隧道对象。
 * \param CrtIdx 隧道对象的下标。
 */
void Sstm::restartTunnelOjbect(int CrtIdx)
{
    Tunnel * tunnel=tunnelList[CrtIdx]; //Get the tunnel object.

    tunnel->kill(); //Kill the tunnel.

    if (tunnel->getAutoRestart()) //This tunnel will auto restart.
    {
    } //if (tunnel->getAutoRestart()) //This tunnel will auto restart.
    else //This tunnel will not auto restart.
    {
        tunnel->start(); //启动对应的隧道。
    } //else //This tunnel will not auto restart.

    tunnelStatusList[CrtIdx]=Running; //记录状态，此隧道在运行。

    ui->startTunnelpushButton->setEnabled (false); //禁用启动按钮。
} //void Sstm::restartTunnelOjbect(int CrtIdx)

/*!
 * \brief Sstm::startTunnelOjbect 启动隧道对象。
 * \param CrtIdx 隧道对象的下标。
 */
void Sstm::startTunnelOjbect(int CrtIdx)
{
    tunnelList[CrtIdx]->start(); //启动对应的隧道。

    tunnelStatusList[CrtIdx]=Running; //记录状态，此隧道在运行。

    ui->startTunnelpushButton->setEnabled (false); //禁用启动按钮。
} //void Sstm::startTunnelOjbect(int CrtIdx)
