#include "SsPreCompile.h" //QObjectHelper

#include "Sstm.h" //Sstm
#include "ui_Sstm.h"
#include "TunnelStatusChecker.h" //TunnelStatusChecker

/*!
 * \brief Tunnel::Tunnel 构造函数。
 * \param parent 亲代对象。
 */
Tunnel::Tunnel(QObject *parent) :QObject(parent)
{
    QTextCodec * gbk_codec=QTextCodec::codecForLocale(); //文字解码器。
    decoder=gbk_codec->makeDecoder(); //获取解码器。
    StdErrdecoder=gbk_codec->makeDecoder(); //获取解码器。


    connectSignals(); //连接信号槽。

    return;
} //Sstm::Sstm(QWidget *parent) :QMainWindow(parent),ui(new Ui::Sstm)

/*!
 * \brief Tunnel::~Tunnel 析构函数。
 */
Tunnel::~Tunnel()
{
    CrtPrc.kill (); //杀死进程。

} //Tunnel::~Tunnel()

/*!
 * \brief Sstm::reportTunnelFinish 报告隧道进程的终止事件。隧道进程结束，则报告状态。
 * \param extCd 退出代码。
 */
void Tunnel::reportTunnelFinish(int extCd)
{
    Q_UNUSED(extCd); //不使用这个参数。

    QProcess * crtPrc=(QProcess *)sender(); //获取信号的发送者。

    QByteArray prcOtptBtAr=crtPrc->readAll (); //全部内容读取。

    QString prcOtpt=QString::fromUtf8 (prcOtptBtAr); //转换成字符串。


    QByteArray prcOtptBtArErr=crtPrc->readAllStandardError (); //全部内容读取。

    QString prcOtptErr=QString::fromUtf8 (prcOtptBtArErr); //转换成字符串。

    prcOtpt+=prcOtptErr; //接到后面。


    QByteArray procStdErrBtAr=crtPrc->readAllStandardError (); //读取全部错误流数据。

    QString prcStdErr=QString::fromUtf8 (procStdErrBtAr); //转换成字符串。

    prcOtpt+=prcStdErr; //追加。


    return;
} //void Sstm::reportTunnelFinish(int extCd) //!<隧道进程结束，则报告状态。

/*!
 * \brief AptF::ShowAptOutputStdErr apt-get有标准错误流输出，则显示。
 */
void Tunnel::ShowAptOutputStdErr()
{

    QProcess * CrtPrc=(QProcess *)sender(); //获取发送数据的进程的指针。

    QString Otpt=StdErrdecoder->toUnicode(CrtPrc->readAllStandardError()); //读入所有输出。

    qDebug() << __FILE__ << __LINE__ << __func__ << Otpt; //显示。


    return;
} //void AptF::ShowAptOutputStdErr() //!<apt-get有标准错误流输出，则显示。


/*!
 * \brief Sstm::reportStdOut 有输出内容，则报告。
 */
void Tunnel::reportStdOut()
{
    QProcess * CrtPrc=(QProcess *)sender(); //获取发送数据的进程的指针。


    while (CrtPrc->bytesAvailable()) //Still has content to read.
    {
        QString Otpt=decoder->toUnicode(CrtPrc->readAllStandardOutput()); //读入所有输出。



             qDebug() << __FILE__ << __LINE__ << __func__ << Otpt; //显示。

        //Get:1 http://jaist.dl.sourceforge.net update/soft qt4-doc 4.8.4-2mgc26 [134MB]
        QRegExp GetRpmReg("Get:[0-9]\\s.*\\s.*\\s(.*)\\s.*\\s\\[[0-9]*(\\.)*[0-9]*M|kB\\]"); //表示当前正在下载RPM包的输出语句。

    } //while (CrtPrc->bytesAvailable()) //Still has content to read.



return;
} //void Sstm::reportStdOut() //!<有输出内容，则报告。

/*!
 * \brief Tunnel::start 启动隧道。
 */
void Tunnel::start()
{
    QStringList Prmtrs; //命令行参数。

    QString dBindAddress=QString("0.0.0.0:%1").arg (CrtLclPrt); //构造D的参数。

    Prmtrs << "-N" << "-D" << dBindAddress << "-p" << QString::number(CrtSrvrPrt) << CrtSrvrIp << "-l" << userName; //加入命令行参数。

    qDebug() << __FILE__ << __LINE__ << __func__ << tr("parameters:") << Prmtrs; //Debug.

    CrtPrc.start(SshCmd,Prmtrs); //启动进程。

    if (autoCheckTunnelStatus) //应当自动检查隧道状态。
    {
        startTunnelStatusChecker(); //启动隧道状态检查器。
    } //if (autoCheckTunnelStatus) //应当自动检查隧道状态。
} //void Tunnel::start()

/*!
 * \brief Tunnel::connectSignals 连接信号槽。
 */
void Tunnel::connectSignals()
{
    connect (&CrtPrc,(void(QProcess::*)(QProcess::ProcessError))&QProcess::error,this,&Tunnel::asesRestart); //进程出错，则评估一下是否要重启。
    connect (&CrtPrc,(void (QProcess::*)(int,QProcess::ExitStatus))&QProcess::finished,this,&Tunnel::reportTunnelFinish); //隧道进程结束，则报告状态。
    connect (&CrtPrc,(void (QProcess::*)(int,QProcess::ExitStatus))&QProcess::finished,this,&Tunnel::asesRestart); //隧道进程结束，则评估一下是否要重启。

    return;
} //void Tunnel::connectSignals()

/*!
 * \brief Tunnel::Tunnel 构造隧道对象。
 * \param CrtSrvrIpToSet 要使用的服务器IP。
 * \param CrtSrvrPrtToSet 要使用的服务器端口号。
 * \param CrtLclPrtToSet 要使用的本地端口号。
 * \param crtUserName 要使用的用户名。
 */
Tunnel::Tunnel(QString CrtSrvrIpToSet,quint16 CrtSrvrPrtToSet,quint16 CrtLclPrtToSet,QString crtUserName):Tunnel()
{
    CrtSrvrIp=CrtSrvrIpToSet; //记录。
    CrtSrvrPrt=CrtSrvrPrtToSet; //记录。
    CrtLclPrt=CrtLclPrtToSet; //记录。
    userName=crtUserName; //记录。
} //Tunnel::Tunnel(QString CrtSrvrIpToSet,quint16 CrtSrvrPrtToSet,quint16 CrtLclPrtToSet,QString crtUserName)

/*!
 * \brief Tunnel::kill 杀死该进程。
 */
void Tunnel::kill ()
{
    qDebug() << __FILE__ << __LINE__ << __func__ ; //Debug.
    CrtPrc.kill (); //杀死进程。

    return;
} //void Tunnel::kill ()

/*!
 * \brief Sstm::asesRestart 进程出错，则评估一下是否要重启。
 * \param errorCode 错误码。
 */
void Tunnel::asesRestart()
{
    qDebug() << __FILE__ << __LINE__ << __func__ ; //Debug.

    if (autoRestart) //应当自动重启。
    {
        start(); //启动隧道对象。
    } //if (autoRestart) //应当自动重启。

    return;
} //void Sstm::asesRestart(QProcess::ProcessError errorCode)

/*!
 * \brief Tunnel::setAutoRestart 设置是否要自动重启。
 * \param autoRestartToSet 是否要自动重启。
 */
void Tunnel::setAutoRestart(bool autoRestartToSet)
{
    autoRestart=autoRestartToSet; //记录。

    return;
} //void Tunnel::setAutoRestart(bool autoRestartToSet)

/*!
 * \brief Tunnel::getLocalPort 获取本地端口号。
 * \return 本地端口号。
 */
quint16 Tunnel::getLocalPort()
{
    return CrtLclPrt;
} //quint16 Tunnel::getLocalPort()

/*!
 * \brief Sstm::startTunnelStatusChecker 启动隧道状态检查器。
 * \param CrtPrc 承载着对应隧道的进程。
 */
void Tunnel::startTunnelStatusChecker()
{
    if (tunnelStatusChecker) //已有隧道状态检查器。
    {
        delete tunnelStatusChecker; //释放内存。
    } //if (tunnelStatusChecker) //已有隧道状态检查器。
    
    tunnelStatusChecker=new TunnelStatusChecker; //创建隧道状态检查器。
    tunnelStatusChecker->setTunnel(this); //设置隧道。

    tunnelStatusChecker->startCheck(); //开始检查。
} //void Tunnel::startTunnelStatusChecker(Tunnel * CrtPrc)

bool Tunnel::getAutoRestart() const
{
    return autoRestart;
}

bool Tunnel::getAutoCheckTunnelStatus() const
{
    return autoCheckTunnelStatus;
}

void Tunnel::setAutoCheckTunnelStatus(bool value)
{
    autoCheckTunnelStatus = value;
}

bool Tunnel::getAutoStart() const
{
    return autoStart;
}

void Tunnel::setAutoStart(bool value)
{
    autoStart = value;
} //void Sstm::startTunnelStatusChecker(QProcess * CrtPrc)

