#include "SsPreCompile.h" //QNetworkReply

#include "TunnelStatusChecker.h"

TunnelStatusChecker::TunnelStatusChecker(QObject *parent) :
    QObject(parent)
{
    initializeMemberObjects(); //初始化成员对象。

    connectSignals(); //连接信号。
}

/*!
 * \brief TunnelStatusChecker::setTunnel 设置要检查的隧道。
 * \param CrtPrc 要检查的隧道。
 */
void TunnelStatusChecker::setTunnel(Tunnel * CrtPrc)
{
    tunnelToCheck=CrtPrc; //记录。

    quint16 localTunnelPort=tunnelToCheck->getLocalPort(); //获取本地端口号。

    setLocalTunnelPort (localTunnelPort); //设置本地端口号。

    return;
} //void TunnelStatusChecker::setTunnel(Tunnel * CrtPrc)

/*!
 *  @brief 开始检查。
 */
void TunnelStatusChecker::startCheck()
{
    statusCheckTimer.start(); //启动测试定时器。

    return;
} //void TunnelStatusChecker::startCheck()

/*!
 * \brief TunnelStatusChecker::initializeMemberObjects 初始化成员对象。
 */
void TunnelStatusChecker::initializeMemberObjects()
{
    statusCheckTimer.setInterval(5*60*1000); //时间间隔是5分钟。


    return;
} //void TunnelStatusChecker::initializeMemberObjects()

/*!
 * \brief TunnelStatusChecker::connectSignals 连接信号。
 */
void TunnelStatusChecker::connectSignals()
{
    connect (&statusCheckTimer,&QTimer::timeout,this,&TunnelStatusChecker::checkTunnelStatus); //定时器超时，则启动一次检查。

    return;
} //void TunnelStatusChecker::connectSignals()

/*!
 * \brief TunnelStatusChecker::checkTunnelStatus 启动一次检查。
 */
void TunnelStatusChecker::checkTunnelStatus()
{
    QString httpPrefix="http://"; //http前缀。瑞哥此刻的婚恋状态是什么？
    QString serverIpPort="stupidbeauty.com"; //服务器IP及端口号。蔡火胜。
    QString commitTextCommand=QString("/robots.txt?%1").arg (QTime::currentTime ().msecsSinceStartOfDay ()); //提交文本的命令。老子。

    QString wholeUrl=httpPrefix+serverIpPort+commitTextCommand; //构造URL。

    qDebug() << __FILE__ << __LINE__ << __func__ << tr("whole url:") << wholeUrl; //Debug.

    QNetworkReply * commitTextReply= networkAccessManager.get(QNetworkRequest(wholeUrl)); //发起请求。

    connect(commitTextReply,&QNetworkReply::finished,this,&TunnelStatusChecker::parseCommitTextReply); //请求结束，则处理结果。
    connect (commitTextReply,static_cast<void (QNetworkReply::*)(const QNetworkReply::NetworkError )>(&QNetworkReply::error),this,&TunnelStatusChecker::reportCommitTextError); //请求出错，则报告出错。

    return;
} //void TunnelStatusChecker::checkTunnelStatus()

/*!
 * \brief SWebPageWidget::parseCommitTextReply 请求结束，则处理结果。
 */
void TunnelStatusChecker::parseCommitTextReply()
{
    QNetworkReply * reply=(QNetworkReply*)sender(); //获取回复对象。

    QByteArray wholeContent=reply->readAll (); //读取全部内容。

    QString wholeContentString=QString::fromUtf8 (wholeContent); //转换成字符串。

    qDebug() << __FILE__ << __LINE__ << __func__ << tr("Whole reply:") << wholeContentString; //Debug.

    return;
} //void SWebPageWidget::parseCommitTextReply()

/*!
 * \brief SWebPageWidget::reportCommitTextError 请求出错，则报告出错。
 * \param code
 */
void TunnelStatusChecker::reportCommitTextError(QNetworkReply::NetworkError code)
{
    QString wholeContentString=tr("Commit text failed:%1").arg (code); //报告错误。

    qDebug() << __FILE__ << __LINE__ << __func__ << wholeContentString; //Debug.

    statusCheckTimer.stop (); //停止检查。

    tunnelToCheck->kill (); //杀死该进程。
} //void SWebPageWidget:: reportCommitTextError(QNetworkReply::NetworkError code)

/*!
 * \brief TunnelStatusChecker::setLocalTunnelPort 设置隧道在本地的端口号。
 * \param localTunnelPort2Set 要设置的端口号。
 */
void TunnelStatusChecker::setLocalTunnelPort(quint16 localTunnelPort2Set)
{

    smtPrxyObj.setType (QNetworkProxy::Socks5Proxy); //socks5代理。
    smtPrxyObj.setHostName ("127.0.0.1"); //本机。
    smtPrxyObj.setPort (localTunnelPort2Set); //端口号。

    networkAccessManager.setProxy (smtPrxyObj); //设置代理。


    return;
} //void TunnelStatusChecker::setLocalTunnelPort(quint16 localTunnelPort2Set)
